/* Internal defenitions for pthreads library.
   Copyright (C) 2000 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#ifndef _PT_SYSDEP_H
#define _PT_SYSDEP_H	1

#include <mach.h>

#include <hurd/threadvar.h>

/* XXX */
#define _POSIX_THREAD_THREADS_MAX	64

/* The default stack size.  */
#define PTHREAD_STACK_DEFAULT	(2 * 1024 * 1024)

#define PTHREAD_SYSDEP_MEMBERS \
  thread_t kernel_thread;      \
  mach_msg_header_t wakeupmsg;

extern inline struct __pthread *
__pthread_self (void)
{
  /* Although we could use hurd_threadvar to do all of the work, this
     turns out to be a bit more complicated than one would like to
     think.  The problem is that after glibc calls
     cthread_init_routine to initialize the thread library, it copies
     all of the variables from the original stack to the new stack
     thereby overwriting what we would have saved.  Here is the fun
     part: we do not know where the base of our stack is if we did not
     create it ourselves.  As such, we cannot set the TSD's the the
     original stack.  To make like easy, we shift the offset of the
     hurd thread variables by one pointer and use the exposed location
     to store the self pointer.  */

  unsigned long int stack_base = ((unsigned long int) __thread_stack_pointer ()
				  & __hurd_threadvar_stack_mask);

  return ((struct __pthread **)
	  (stack_base + __hurd_threadvar_stack_offset))[_HURD_THREADVAR_MAX];
}

#define _pthread_self() \
	({ struct __pthread *t = __pthread_self (); \
	   assert (t->kernel_thread == __mach_thread_self ()); \
	   t; })

extern inline void
__pthread_stack_dealloc (void *stackaddr, size_t stacksize)
{
  __vm_deallocate (__mach_task_self (), (vm_offset_t) stackaddr, stacksize);
}

/* Change thread THREAD's program counter to PC if SET_PC is true and
   its stack pointer to SP if SET_IP is true.  */
extern int __thread_set_pcsp (thread_t thread,
			      int set_pc, void *pc,
			      int set_sp, void *sp);


#endif /* pt-sysdep.h */
